"""Maintain indices for photo collections

This package maintains indices for photo collections.  The index is
stored as a YAML file and contains metadata and tags describing the
photos.  The photos are accessed read only.
"""

__version__ = "0.10.1"

